"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformStreamRow = exports.transformRow = void 0;
const moment = __importStar(require("moment"));
//  ClickHouse returns DateTime as strings in format "YYYY-DD-MM HH:MM:SS"
//  cube.js expects them in format "YYYY-DD-MMTHH:MM:SS.000", so translate them based on the metadata returned
//
//  ClickHouse returns some number types as js numbers, others as js string, normalise them all to strings
function transformValue(type, value) {
    if (value !== null) {
        if (type.includes('DateTime64')) {
            return moment.utc(value).format(moment.HTML5_FMT.DATETIME_LOCAL_MS);
        }
        else if (type.includes('DateTime') /** Can be DateTime or DateTime('timezone') */) {
            // expect DateTime to always be string
            const valueStr = value;
            return `${valueStr.substring(0, 10)}T${valueStr.substring(11, 22)}.000`;
        }
        else if (type.includes('Date')) {
            return `${value}T00:00:00.000`;
        }
        else if (type.includes('Int')
            || type.includes('Float')
            || type.includes('Decimal')) {
            // convert all numbers into strings
            return `${value}`;
        }
    }
    return value;
}
function transformRow(row, meta) {
    for (const [fieldName, value] of Object.entries(row)) {
        const metaForField = meta[fieldName];
        row[fieldName] = transformValue(metaForField.type, value);
    }
}
exports.transformRow = transformRow;
function transformStreamRow(row, names, types) {
    if (row.length !== names.length) {
        throw new Error(`Unexpected row and names/types length mismatch; row ${row.length} vs names ${names.length}`);
    }
    return row.reduce((rowObj, value, idx) => {
        const name = names[idx];
        const type = types[idx];
        rowObj[name] = transformValue(type, value);
        return rowObj;
        // TODO do we actually want Object.create(null) safety? or is it ok to use {}
    }, Object.create(null));
}
exports.transformStreamRow = transformStreamRow;
//# sourceMappingURL=HydrationStream.js.map